<?php

/**
 * Product Title:		Pay.ir gateway
 * Product Version:		1.0
 * Author:				Pay.ir team
 * Website URL:			https://pay.ir
 * Email:				info@pay.ir
 */

if ( ! defined( 'GW_CORE_INIT' ) )
{
	print "You cannot access this module in this manner";
	exit();
}

//--------------------------------------------------------------------------
// DEFINITIONS EXPECTED AT THIS POINT
//--------------------------------------------------------------------------
// GW_URL_VALIDATE : The url for validating payment
// GW_URL_PAYDONE  : The url that the gatways returns the viewer to after
//                 : payment processed successfully
// GW_URL_PAYCANCEL: The url that the gatways returns the viewer to after
//                 : payment processed unsuccessfully or when cancelled
//--------------------------------------------------------------------------
// ITEM ARRAY
//--------------------------------------------------------------------------
// 'currency_code'    => Currency code,
// 'member_unique_id' => member's ID,
// 'member_name'      => member's NAME,
// 'member_email'     => member's EMAIL,
// 'package_cost'     => Requested package cost
// 'package_id'       => Requested package ID
// 'package_title'    => Requested package title
// 'duration_int'     => Requested package duration int  (ie: 12)
// 'duration_unit'    => Requested package duration unit (ie: m,d,y,w) [ month, day, year, week ]
// 'company_email'    => Company's email address
// 'ttr_int'          => Time to run (Time left on current package) integar (ie 3)
// 'ttr_unit'         => Time to run (Time left on current package) unit (ie w)
// 'ttr_balance'      => Time to run (Balance left on current package)
// 'ttr_package_id'   => Current package id (used for upgrading)
//--------------------------------------------------------------------------

class gatewayApi_payir extends apiCore
{
	/**
	 * Identify this class
	 *
	 * @access	public
	 * @var		string
	 */
	const API_NAME = 'payir';
	
	/**
	 * Can I do recurring billing?
	 *
	 * @access	 public
	 * @var		 boolean
	 */
	public $ALLOW_RECURRING = TRUE;
	
	/**
	 * Can I do upgrades?
	 *
	 * @access	 public
	 * @var		 boolean
	 */
	public $ALLOW_UPGRADES = TRUE;
	
	/**
	 * Can I post back?
	 *
	 * @access	 public
	 * @var		 boolean
	 */
	public $FORBID_POSTBACK = TRUE;
	
	
	public $item = array();
	
	
	function __construct( ipsRegistry $registry )
	{
		parent::__construct( $registry );
	}
	
	
	/*-------------------------------------------------------------------------*/
	// Generate hidden fields [ Recurring, normal screen ]
	/*-------------------------------------------------------------------------*/
	
	function makeFields_normal_recurring( $items=array() )
	{
		$this->item = $items;
		return $this->compileFields();
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate hidden fields [ Recurring, upgrade screen ]
	/*-------------------------------------------------------------------------*/
	
	function makeFields_upgrade_recurring( $items=array() )
	{
		$this->item = $items;
		return $this->compileFields();
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate hidden fields [ normal screen ]
	/*-------------------------------------------------------------------------*/
	
	function makeFields_normal( $items=array() )
	{
		$this->item = $items;
		return $this->compileFields();
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate hidden fields [ upgrade screen ]
	/*-------------------------------------------------------------------------*/
	
	function makeFields_upgrade( $items=array() )
	{
		$this->item = $items;
		return $this->compileFields();
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate Purchase button
	/*-------------------------------------------------------------------------*/
	
	function makePurchaseButton()
	{
		return '<input type="submit" class="input_submit" value="پرداخت" />';
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate Form action [normal]
	/*-------------------------------------------------------------------------*/

	function common($url, $params)
	{
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));

		$response = curl_exec($ch);
		$error    = curl_errno($ch);

		curl_close($ch);

		$output = $error ? FALSE : json_decode($response);

		return $output;
	}
	
	function makeFormAction_normal()
	{
		if (extension_loaded('curl')) {

			$api_key = $this->item['vendor_id'];

			if (empty($api_key)) {

				$message = 'کلید API برای اتصال به درگاه پرداخت یافت نشد';

				ipsRegistry::getClass('output')->showError($message, 'API', FALSE, NULL, 403);
			}

			$callback = GW_URL_VALIDATE . '&tid=' . $this->item['transaction_id'] . '&verification=';
			$callback = $callback . md5($this->item['member_unique_id'] . $this->item['package_id'] . $this->settings['sql_pass']);

			$amount = floor($this->item['package_cost']);
			$title  = $this->item['package_title'];

			$params = array(

				'api'          => $api_key,
				'amount'       => $amount,
				'redirect'     => urlencode($callback),
				'factorNumber' => $this->item['transaction_id']
			);

			$result = $this->common('https://pay.ir/payment/send', $params);

			if ($result && isset($result->status) && $result->status == 1) {

				@session_start();
				$_SESSION['transId'] = $result->transId;

				return 'https://pay.ir/payment/gateway/' . $result->transId;

			} else {

				$message = 'در ارتباط با وب سرویس Pay.ir خطایی رخ داده است';
				$message = isset($result->errorMessage) ? $result->errorMessage : $message;

				ipsRegistry::getClass('output')->showError($message, 'Request', FALSE, NULL, 403);
			}

		} else {

			$message = 'تابع cURL در سرور فعال نمی باشد';

			ipsRegistry::getClass('output')->showError($message, 'cURL', FALSE, NULL, 403);
		}
	}

	/*-------------------------------------------------------------------------*/
	// Generate Form action [upgrade]
	/*-------------------------------------------------------------------------*/
	
	function makeFormAction_upgrade()
	{
		return $this->makeFormAction_normal();
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate Form action [normal, recurring]
	/*-------------------------------------------------------------------------*/
	
	function makeFormAction_normal_recurring()
	{
		return $this->makeFormAction_normal();
	}
	
	/*-------------------------------------------------------------------------*/
	// Generate Form action [upgrade, recurring]
	/*-------------------------------------------------------------------------*/
	
	function makeFormAction_upgrade_recurring()
	{
		return $this->makeFormAction_normal();
	}
	
	/*-------------------------------------------------------------------------*/
	// Validate Payment
	// What we need to return:
	// 'currency_code'      => Currency code,
	// 'payment_amount'     => Amount paid,
	// 'payment_status'     => REFUND, ONEOFF, RECURRING
	// 'member_unique_id'   => member's ID,
	// 'purchase_package_id'=> Purchased package ID
	// 'current_package_id' => Current package ID (used for upgrading)
	// 'verified'           => TRUE , FALSE (Gateway verifies info as correct)
	// 'subscription_id'    => (Used for recurring payments)
	// 'transaction_id'     => Gateway transaction ID
	/*-------------------------------------------------------------------------*/
	
	function validatePayment($sets=array())
	{
		if (isset($this->request['status']) && isset($this->request['transId']) && isset($this->request['factorNumber'])) {

			$api_key = $sets['vendor_id'];

			if (empty($api_key)) {

				$message = 'کلید API برای اتصال به درگاه پرداخت یافت نشد';

				ipsRegistry::getClass('output')->showError($message, 'API', FALSE, NULL, 403);
				return $this->showerror();
			}

			$status        = $this->request['status'];
			$trans_id      = $this->request['transId'];
			$factor_number = $this->request['factorNumber'];
			$message       = $this->request['message'];

			if (isset($status) && $status == 1) {

				$trans = $this->DB->buildAndFetch(array(

					'select' => '*', 'from' => 'subscription_trans', 'where' => "subtrans_id='{$factor_number}'"
				));
		
				if ($trans['subtrans_id']) {

					$params = array (

						'api'     => $api_key,
						'transId' => $trans_id
					);

					$result = $this->common('https://pay.ir/payment/verify', $params);

					@session_start();

					if ($result && isset($result->status) && $result->status == 1) {

						if ($_SESSION['transId'] == $trans_id) {

							$amount = floor($trans['subtrans_to_pay']);
							
							if ($amount == $result->amount) {

								$return = array(

									'currency_code'      => trim('IRR'),
									'payment_amount'     => $amount,
									'member_unique_id'   => $trans['subtrans_member_id'],
									'subtrans_id'		 => $trans['subtrans_id'],
									'purchase_package_id'=> $trans['subtrans_sub_id'],
									'current_package_id' => '',
									'verified'           => TRUE,
									'verification'		 => $this->request['verification'],
									'subscription_id'    => '0-' . intval($trans['subtrans_member_id']),
									'transaction_id'     => $trans_id,
									'renewing'			 => 0,
									'payment_status'     => 'ONEOFF',
									'state'              => 'paid',
								);
			
								return $return;

							} else {
							
								$message = 'رقم تراكنش با رقم پرداخت شده مطابقت ندارد';

								ipsRegistry::getClass('output')->showError($message, 'Amount', FALSE, NULL, 403);
								return $this->showerror();
							}

						} else {

							$message = 'در حین انجام تراکنش خطایی رخ داده است';

							ipsRegistry::getClass('output')->showError($message, 'Wrong', FALSE, NULL, 403);
							return $this->showerror();
						}

					} else {
						
						$message = 'در ارتباط با وب سرویس Pay.ir و بررسی تراکنش خطایی رخ داده است';
						$message = isset($result->errorMessage) ? $result->errorMessage : $message;

						ipsRegistry::getClass('output')->showError($message, 'Error', FALSE, NULL, 403);
						return $this->showerror();
					}

				} else {

					$message = 'این ترانش در بانک اطلاعاتی موجود نیست';

					ipsRegistry::getClass('output')->showError($message, 'Notfound', FALSE, NULL, 403);
					return $this->showerror();
				}

			} else {

				if ($message) {

					ipsRegistry::getClass('output')->showError($message, $status, FALSE, NULL, 403);
									
				} else {

					$message = 'تراكنش با خطا مواجه شد و یا توسط پرداخت کننده کنسل شده است';

					ipsRegistry::getClass('output')->showError($message, 'Invalid', FALSE, NULL, 403);
				}

				return $this->showerror();
			}

		} else {

			$message = 'اطلاعات ارسال شده مربوط به تایید تراکنش ناقص و یا غیر معتبر است';

			ipsRegistry::getClass('output')->showError($message, 'Data', FALSE, NULL, 403);
			return $this->showerror();
		}
	}
	
	public function showerror()
	{
		$this->error = 'not_valid';

		return array(

			'verified' => FALSE
		);
	}
	
	//---------------------------------------
	// Return ACP Package  Variables
	//
	// Returns names for the package custom
	// fields, etc
	//---------------------------------------
	
	function acpReturnPackageData()
	{
		return array( 'subextra_custom_1' => array( 'used' => 0, 'varname' => '' ),
					  'subextra_custom_2' => array( 'used' => 0, 'varname' => '' ),
					  'subextra_custom_3' => array( 'used' => 0, 'varname' => '' ),
					  'subextra_custom_4' => array( 'used' => 0, 'varname' => '' ),
					  'subextra_custom_5' => array( 'used' => 0, 'varname' => '' ),
					 );
	}
	
	//---------------------------------------
	// Return ACP Method Variables
	//
	// Returns names for the package custom
	// fields, etc
	//---------------------------------------

	function acpReturnMethodData()
	{
		return array( 'submethod_custom_1' => array( 'used' => 0, 'varname' => '' ),
					  'submethod_custom_2' => array( 'used' => 0, 'varname' => '' ),
					  'submethod_custom_3' => array( 'used' => 0, 'varname' => '' ),
					  'submethod_custom_4' => array( 'used' => 0, 'varname' => '' ),
					  'submethod_custom_5' => array( 'used' => 0, 'varname' => '' ),
					);
	}

	/*-------------------------------------------------------------------------*/
	// INSTALL ROUTINES
	/*-------------------------------------------------------------------------*/

	function acpInstallGateway()
	{
		$this->db_info = array( 'human_title'         => 'Payir',
								'human_desc'		  => 'Payir gateway for iranian.',
								'module_name'         => self::API_NAME,
								'allow_creditcards'   => 1,
								'allow_auto_validate' => 1,
								'default_currency'    => 'IRR' );
																   
		$this->install_lang = array( 'payir_novender' => 'Please report to admin' );
	}
}